from datetime import date
import json
import os
# from .models import CastingCallApplication
from django.contrib.auth import get_user_model
from save_app.models import SavedApprenticeship, SavedArtist, SavedGigWork, SavedInternship
from rest_framework import serializers
from .models import ApprenticeApplication, Credit, GigWork, CastingCall, Headshot, InternshipApplication, MediaRelease, PressRelease, Project,TeamMember,Award
from rest_framework import serializers
from .models import Internship, Apprenticeship, EventRegistration, BankDetails, Uploads, EventDetails,Feedback,ProjectComment,ProjectReport
from.models import SavedProject,FollowedProject,Team,CastingCallApplication,GigWorkApplication
from userprofile_app.models import City, Profile, Skill, State
from django.shortcuts import get_object_or_404
CustomUser = get_user_model()

class SuggestedUserSerializer(serializers.ModelSerializer):
    # selected_primary_industry = serializers.CharField(source='selected_primary_industry.name')  # Primary industry name
    selected_primary_industry = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()
    userimage = serializers.ImageField(source='profile_image')  # Image field mapping
    artist_save_status = serializers.SerializerMethodField()
    location = serializers.SerializerMethodField()
    age= serializers.SerializerMethodField()

    class Meta:
        model = Profile
        fields = ['id', 'user_id', 'selected_primary_industry', 'userimage','full_name','artist_save_status','location','age']
        
    def get_selected_primary_industry(self, obj):
        return obj.selected_primary_industry.name if obj.selected_primary_industry else None
    
    def get_full_name(self, obj):
        return obj.user.full_name
    
    def get_artist_save_status(self, obj):
        request = self.context.get('request')
        if request and hasattr(request, 'user'):
            return SavedArtist.objects.filter(user_id=request.user.id, artist_profile_id=obj.id).exists()
        return False
    def get_location(self, obj):
        if obj.user.profile.location:
            # city_id, state_id = map(int, obj.user.profile.location.split(','))
            parts = obj.user.profile.location.split(',')
            if len(parts) != 2 or not all(part.strip().isdigit() for part in parts):
                return "Unknown Location"

            city_id, state_id = map(int, parts)
            try:
                city_name = City.objects.get(id=city_id).name
                state_name = State.objects.get(id=state_id).name
                return f"{city_name}, {state_name}"
            except City.DoesNotExist:
                city_name = "Unknown City"
            except State.DoesNotExist:
                state_name = "Unknown State"
            return f"{city_name}, {state_name}"
        return None
    def get_age(self, obj):
        return obj.user.profile.age
    
class CollabUserSerializer(serializers.ModelSerializer):
    # selected_primary_industry = serializers.CharField(source='selected_primary_industry.name')  # Primary industry name
    selected_primary_industry = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()
    username = serializers.SerializerMethodField()
    looking_for = serializers.SerializerMethodField()
    userimage = serializers.ImageField(source='profile_image')  # Image field mapping
    coverimage = serializers.ImageField(source='cover_image')
    selected_skills = serializers.SerializerMethodField()

    class Meta:
        model = Profile
        fields = ['id', 'user_id','username', 'selected_primary_industry', 'userimage','full_name','looking_for', 'selected_skills','coverimage']
        
    def get_selected_primary_industry(self, obj):
        return obj.selected_primary_industry.name if obj.selected_primary_industry else None
    
    def get_full_name(self, obj):
        return obj.user.full_name
    
    def get_username(self, obj):
        return obj.user.username
    
    def get_looking_for(self, obj):
        return dict(Profile.LOOKING_FOR_CHOICES).get(obj.looking_for, "None")  # Converts the code to a readable string

    def get_selected_skills(self, obj):
        # Returns a list of skill names
        return list(obj.selected_skills.values_list('name', flat=True))

class GigWorkSerializer(serializers.ModelSerializer):
    selected_project_id = serializers.IntegerField(required=False, allow_null=True, write_only=True)
    # user_id = serializers.SerializerMethodField()
    user_id = serializers.IntegerField(source='user.id', read_only=True)
    full_name = serializers.SerializerMethodField()
    profile_image = serializers.SerializerMethodField()
    skill_names = serializers.SerializerMethodField()
    project_name = serializers.SerializerMethodField()
    language = serializers.SerializerMethodField()
    save_status = serializers.SerializerMethodField()
    applied_status = serializers.SerializerMethodField()
    age = serializers.SerializerMethodField()

    class Meta:
        model = GigWork
        fields = '__all__'

    def validate(self, data):
        gigwork_type = data.get('gigworks_type')
        project_id = data.get('selected_project_id')

        if gigwork_type == 'project' and not project_id:
            raise serializers.ValidationError({
                'selected_project': 'Project ID is required when gigworks_type is "Project".'
            })
        if gigwork_type == 'individual' and project_id in [0, None]:
            data['selected_project_id'] = None
        return data

    def get_age(self, obj):
        """Fetch the age of the user who created the selected project."""
        if obj.selected_project and obj.selected_project.user:
            profile = getattr(obj.selected_project.user, 'profile', None)
            return profile.age if profile and profile.age is not None else None
        return None

    def get_save_status(self, obj):
        user = self.context.get('request').user
        return SavedGigWork.objects.filter(gig_work = obj.id, user_id = user.id ).exists()

    def get_applied_status(self, obj):
        user = self.context.get('request').user
        return GigWorkApplication.objects.filter(gigwork_id = obj.id, user = user.id).exists()

    def get_user_id(self, obj):
        return obj.selected_project.user.id if obj.selected_project and obj.selected_project.user else None

    def get_profile_image(self, obj):
        profile = getattr(obj.user, 'profile', None)
        return profile.profile_image.url if profile and profile.profile_image else None

    def get_full_name(self, obj):
        return obj.user.full_name

    # def get_profile_image(self, obj):
    #     profile = getattr(obj.selected_project.user, 'profile', None) if obj.selected_project else None
    #     return profile.profile_image.url if profile and profile.profile_image else None
    
    def get_skill_names(self, obj):
        if not obj.skills:
            return []
        skill_ids = obj.skills
        skills = Skill.objects.filter(id__in=skill_ids)
        return [skill.name for skill in skills]
    
    def get_project_name(self, obj):
        if hasattr(obj, "selected_project") and obj.selected_project:
            return getattr(obj.selected_project, "project_title", None)
        return None
    
    def get_language(self, obj):  # New method to fetch language
        if hasattr(obj, "selected_project") and obj.selected_project:
            return getattr(obj.selected_project, "language", None)
        return None
    
class GigWorkApplicationSerializer(serializers.ModelSerializer):
    gigwork_details = GigWorkSerializer(source='gigwork', read_only=True)
    class Meta:
        model = GigWorkApplication
        fields = ['gigwork', 'user_message', 'attachments','gigwork_details']
        read_only_fields = ['user'] 
    
class GigworkStatusSerializer(serializers.ModelSerializer):
    class Meta:
        model = GigWork
        fields = ['status']

    def validate_status(self, value):
        if value not in dict(GigWork.STATUS_CHOICES).keys():
            raise serializers.ValidationError("Invalid status.")
        return value
    
class GigWorkApplicationStatusUpdateSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = GigWorkApplication
        fields = ['status']

    def validate_status(self, value):
        valid_statuses = ['applied', 'selected', 'waiting_list', 'save_later', 'rejected']
        if value not in valid_statuses:
            raise serializers.ValidationError("Invalid status value.")
        
        instance = getattr(self, 'instance', None)
        if instance:
            if instance.status in ['selected', 'rejected']:
                raise serializers.ValidationError(f"Status '{instance.status}' cannot be changed once set.")
        return value


class TeamJoinRequestSerializer(serializers.Serializer):
    user_id = serializers.IntegerField()
    casting_call_id = serializers.IntegerField()
    team_id = serializers.IntegerField()

# class TeamSerializer(serializers.ModelSerializer):
#     created_by_username = serializers.CharField(source='created_by.username', read_only=True)

#     class Meta:
#         model = Team
#         fields = '__all__'
#         read_only_fields = ['created_by']

# class TeamMemberSerializer(serializers.ModelSerializer):
#     added_by_username = serializers.CharField(source='added_by.username', read_only=True)

#     class Meta:
#         model = TeamMember
#         fields = '__all__'
#         read_only_fields = ['added_by']


class TeamMemberSerializer(serializers.ModelSerializer):
    class Meta:
        model = TeamMember
        fields = ["user","name", "role"]



class TeamSerializer(serializers.ModelSerializer):
    class Meta:
        model = Team
        fields = ['id', 'team_name', 'project']  # Include relevant fields
        read_only_fields = ['id']


class CastingCallApplicationSerializer(serializers.ModelSerializer):
    # team_id = serializers.PrimaryKeyRelatedField( queryset=Team.objects.all(), required=False, allow_null=True )
    class Meta:
        model = CastingCallApplication
        fields = '__all__'
        read_only_fields = ['status', 'applied_at']


class AwardSerializer(serializers.ModelSerializer):
    class Meta:
        model = Award
        fields = '__all__'


class ProjectAwardsSerializer(serializers.ModelSerializer):
    awards = serializers.ListField(
        child=serializers.CharField(max_length=255), allow_empty=True
    )

    class Meta:
        model = Project
        fields = ['id', 'awards']

class UserSerializer(serializers.ModelSerializer):
    """
    Serializer to extract user data from the application instance.
    """
    class Meta:
        model = CustomUser
        fields = ['id', 'username', 'email']


class TeamMemberCreditRemoveSerializer(serializers.ModelSerializer):
    class Meta:
        model = TeamMember
        fields = ['role']

class ProjectPhotoUploadSerializer(serializers.ModelSerializer):
    press_releases = serializers.ListField(
        child=serializers.ImageField(allow_empty_file=False, use_url=True), required=True
    )

    class Meta:
        model = Project
        fields = ['press_releases']

    def create(self, validated_data):
        images = validated_data.get('press_releases', [])
        project = validated_data.get('project')  # Ensure you get the related project instance

        # Create PressRelease instances for each uploaded image
        press_releases = []
        for image in images:
            press_release = PressRelease.objects.create(
                project=project,
                image=image
            )
            press_releases.append(press_release)

        # Add the newly created PressRelease instances to the project
        project.press_releases.add(*press_releases)
        project.save()

        return project



class ProjectVideoUploadSerializer(serializers.ModelSerializer):
    media_releases = serializers.ListField(
        child=serializers.FileField(allow_empty_file=False, use_url=True),
        required=False
    )

    class Meta:
        model = Project
        fields = ['media_releases']

    def update(self, instance, validated_data):
        videos = validated_data.get('media_releases', [])
        if videos:
            instance.media_releases.extend(videos)  # Assuming it's a JSON field or list
            instance.save()
        return instance

# class ProjectPhotoUploadSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = Project
#         fields = ['press_release']

# class ProjectVideoUploadSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = Project
#         fields = ('media_release',)



class FollowProjectSerializer(serializers.ModelSerializer):
    class Meta:
        model = FollowedProject
        fields = ('user', 'project', 'followed_at')
        read_only_fields = ('followed_at',)


class SaveProjectSerializer(serializers.ModelSerializer):
    class Meta:
        model = SavedProject
        fields = ['user', 'project']
        read_only_fields = ['user']


class ProjectReportSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProjectReport
        fields = ['reason', 'description', 'is_blocked']


class ProjectTitleSerializer(serializers.ModelSerializer):
    class Meta:
        model = Project
        fields = ['id','project_title']
    
class PressReleaseSerializer(serializers.ModelSerializer):
    class Meta:
        model = PressRelease
        fields = ['id', 'image', 'uploaded_at']

class MediaReleaseSerializer(serializers.ModelSerializer):
    class Meta:
        model = MediaRelease
        fields = ['id', 'file', 'uploaded_at']

class ProjectSerializer(serializers.ModelSerializer):
    project_id = serializers.IntegerField(source='id', read_only=True)
    thumbnail_url = serializers.SerializerMethodField()  # Add custom field for image URL
    cover_image_url = serializers.SerializerMethodField()  # Add custom field for video URL
    # user_id = serializers.IntegerField(source='user.id', read_only=True)
    profile_image = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()
    save_status = serializers.SerializerMethodField()
    press_releases = PressReleaseSerializer(many=True, read_only=True)
    media_releases = MediaReleaseSerializer(many=True, read_only=True)


    class Meta:
        model = Project
        fields = [
            'project_id','user_id','full_name', 'project_title', 'project_tagline', 'character_name', 'role_played',
            'project_status', 'progress_percentage', 'project_type', 'language', 'description',
            'primary_email', 'secondary_email','project_name',
            'hashtags', 'press_releases', 'media_releases', 'social_media_links','cover_image',
            'genre', 'created_at', 'thumbnail_url', 'cover_image_url', 'status', 'profile_image' ,'save_status','start_date','end_date'
        ]

    def get_save_status(self, obj):
        user = self.context.get('request').user
        return SavedProject.objects.filter(project = obj.id, user_id = user.id ).exists()


    def get_full_name(self, obj):
        return obj.user.full_name

    def get_thumbnail_url(self, obj):
       return obj.thumbnail_image.url if obj.thumbnail_image else None

    def get_cover_image_url(self, obj):
        return obj.cover_image.url if obj.thumbnail_image else None

    def create(self, validated_data):
        user = self.context['request'].user  # Get the user from the request
        project = Project.objects.create(user=user,**validated_data)
        return project
    
    
    def validate_awards(self, value):
        if not isinstance(value, list):
            raise serializers.ValidationError("Value must be valid JSON array of awards.")
        return value
    
    def get_profile_image(self, obj):
        # profile_image =  obj.user.profile.profile_image
        profile = getattr(obj.user, 'profile', None)
        return profile.profile_image.url if profile and profile.profile_image else None
        # return profile_image


class CreateProjectSerializer(serializers.ModelSerializer):
    press_releases = PressReleaseSerializer(many=True, required=False)
    media_releases = MediaReleaseSerializer(many=True, required=False)
    class Meta:
        model = Project
        fields = [
            'id', 'project_title', 'project_tagline', 'character_name', 'role_played', 
            'project_status', 'progress_percentage', 'project_type', 'language', 
            'description', 'primary_email', 'secondary_email', 'hashtags', 'project_name',
            'press_releases', 'media_releases', 'social_media_links', 'genre',
            'created_at', 'user_id', 'cover_image', 'status', 'thumbnail_image','start_date','end_date'
        ]

    def validate_hashtags(self, value):
        """ Convert hashtags to a list if given as a string. """
        if isinstance(value, str):
            return [tag.strip() for tag in value.split(",") if tag.strip()]
        return value or []  # Ensure it's always a list

    def validate_social_media_links(self, value):
        """ Convert social media links to a list if given as a string. """
        if isinstance(value, str):
            try:
                return json.loads(value)  # Convert JSON string to list
            except json.JSONDecodeError:
                return [value]  # Treat as single URL in a list
        return value or []  # Ensure it's always a list

    def create(self, validated_data):
        request = self.context.get('request')

        hashtags = request.data.get("hashtags", "")
        social_media_links = request.data.get("social_media_links", "")

        validated_data['hashtags'] = self.validate_hashtags(hashtags)
        validated_data['social_media_links'] = self.validate_social_media_links(social_media_links)
        
        press_releases_data = validated_data.pop('press_releases', [])
        media_releases_data = validated_data.pop('media_releases', [])

        project = Project.objects.create(user=request.user, **validated_data)

        for press_release_data in press_releases_data:
            PressRelease.objects.create(project=project, **press_release_data)

        for media_release_data in media_releases_data:
            MediaRelease.objects.create(project=project, **media_release_data)

        return project

    # def create(self, validated_data):
    #     # Attach the authenticated user to the project
    #     request = self.context.get('request')
    #     validated_data['user'] = request.user
    #     return super().create(validated_data)
    
    # def validate_social_media_links(self, value):
    #     if not isinstance(value, list):
    #         raise serializers.ValidationError("social_media_links must be provided as a list.")
    #     return value


class ProjectUpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Project
        fields = [
            "project_title",
            "project_tagline",
            'character_name',
            'role_played',
            "project_status",
            "progress_percentage",
            "project_type",
            "language",
            "description",
            "primary_email",
            "secondary_email",
            "image_files",
            "video_files",
            'start_date','end_date',
        ]

class CommentSerializer(serializers.ModelSerializer):
    user_name = serializers.CharField(source='user.username', read_only=True)
    full_name = serializers.SerializerMethodField()
    project = serializers.PrimaryKeyRelatedField(queryset=Project.objects.all())
    role = serializers.SerializerMethodField() 
    class Meta:
        model = ProjectComment
        fields = ['id', 'project','user', 'content','user_name','full_name','role', 'created_at']

    def get_full_name(self, obj):
        return obj.user.full_name
    
    def get_role(self, obj):
        # Fetch the user's role in the project
        credit = Credit.objects.filter(project=obj.project, user=obj.user).first()
        return credit.role if credit else "No Role Assigned"


class BankDetailsSerializer(serializers.ModelSerializer):
    class Meta:
        model = BankDetails
        fields = '__all__'

class UploadsSerializer(serializers.ModelSerializer):
    pan_card = serializers.FileField()
    cancelled_cheque = serializers.FileField()

    class Meta:
        model = Uploads
        fields = ['pan_card', 'cancelled_cheque']

    def validate(self, attrs):
        """Ensure uploaded files have valid names and sanitize them."""
        for field_name, file in attrs.items():
            if file:
                # Validate that the file name doesn't contain backslashes or other invalid characters
                if '\\' in file.name:
                    raise serializers.ValidationError(
                        f"File name for {field_name} must not contain '\\'."
                    )
                
                # Sanitize the file name
                sanitized_name = os.path.basename(file.name).replace('\\', '')
                if sanitized_name != file.name:
                    file.name = sanitized_name  # Assign sanitized name back to the file

                # Additional validation for invalid characters (optional)
                invalid_chars = ['/', ':', '*', '?', '"', '<', '>', '|']
                if any(char in sanitized_name for char in invalid_chars):
                    raise serializers.ValidationError(
                        f"File name for {field_name} contains invalid characters: {', '.join(invalid_chars)}."
                    )

        return attrs

class EventDetailsSerializer(serializers.ModelSerializer):
    class Meta:
        model = EventDetails
        fields = '__all__'

class EventRegistrationSerializer(serializers.ModelSerializer):
    bank_details = BankDetailsSerializer()
    uploads = UploadsSerializer()
    event_details = EventDetailsSerializer()

    class Meta:
        model = EventRegistration
        exclude = ['user']  # Exclude the user field

    def create(self, validated_data):
        request = self.context['request']

        bank_data = validated_data.pop('bank_details')
        uploads_data = validated_data.pop('uploads')
        event_data = validated_data.pop('event_details')

        # Create related instances
        bank_details = BankDetails.objects.create(**bank_data)
        uploads = Uploads.objects.create(**uploads_data)
        event_details = EventDetails.objects.create(**event_data)

        # Create event registration with related data and user
        event_registration = EventRegistration.objects.create(
            user=request.user,
            bank_details=bank_details,
            uploads=uploads,
            event_details=event_details,
            **validated_data
        )
        return event_registration

    def update(self, instance, validated_data):
        # Update the instance fields
        bank_data = validated_data.pop('bank_details', None)
        uploads_data = validated_data.pop('uploads', None)
        event_data = validated_data.pop('event_details', None)

        # Update fields directly on the instance
        for attr, value in validated_data.items():
            setattr(instance, attr, value)
        instance.save()

        # Update related instances if provided
        if bank_data:
            for key, value in bank_data.items():
                setattr(instance.bank_details, key, value)
            instance.bank_details.save()

        if uploads_data:
            for key, value in uploads_data.items():
                setattr(instance.uploads, key, value)
            instance.uploads.save()

        if event_data:
            for key, value in event_data.items():
                setattr(instance.event_details, key, value)
            instance.event_details.save()

        return instance


class InternshipSerializer(serializers.ModelSerializer):
    class Meta:
        model = Internship
        fields = [
            'name', 'designation', 'internship_documents',
            'email', 'mobile_number', 'organisation_name',
            'intern_method', 'internship_title', 'skills_list', 
            'internship_type', 'type_of_work', 'start_date', 
            'end_date', 'job_responsibilities', 'duration'
        ] 

class ApprenticeshipSerializer(serializers.ModelSerializer):
    class Meta:
        model = Apprenticeship
        fields = [
            'name', 'designation', 'apprenticeship_documents',
            'email', 'mobile_number', 'organisation_name',
            'apprenticeship_method', 'apprenticeship_title', 
            'skills_list', 'apprenticeship_type', 'type_of_work', 
            'start_date', 'end_date', 'job_responsibilities', 'duration'
        ]

class CastingCallSerializer(serializers.ModelSerializer):
    project_id = serializers.IntegerField(required=False, allow_null=True, write_only=True)
    script_link = serializers.CharField(required=False, allow_null=True, allow_blank=True)
    user_id = serializers.IntegerField(source='user.id', read_only=True)
    profile_image = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()
    body_type = serializers.CharField(required=False, allow_null=True, allow_blank=True)
    height = serializers.DecimalField(max_digits=5, decimal_places=2, required=False, allow_null=True)
    profile_age = serializers.SerializerMethodField()
    username = serializers.SerializerMethodField()
    class Meta:
        model = CastingCall
        fields = [
            'id','username','user_id', 'profile_image', 'full_name','project_id', 'user', 'job_title', 'select_project', 'short_description',
            'gender', 'experience', 'role_type', 'age', 'skills', 'height',
            'body_type', 'upload_photos', 'url_link', 'audition_type', 'salary','castingcall_type',
            'castingcall_script', 'script_link', 'location', 'start_date', 
            'end_date', 'start_time', 'end_time', 'created_at','profile_age','thumbnail','logo','castingcall_type',
        ]

    def validate(self, data):
        castingcall_type = data.get('castingcall_type')
        project_id = data.get('project_id')

        if castingcall_type == 'project' and not project_id:
            raise serializers.ValidationError({
                'project_id': 'Project ID is required when castingcall_type is "Project".'
            })
        
        if castingcall_type == 'individual' and project_id in [0, None]:
            data['project_id'] = None

        return data
    
    def get_username(self, obj):
        return obj.user.username
    
    def get_profile_age(self, obj):
        profile = getattr(obj.user, 'profile', None)
        if profile and profile.age is not None:
            return profile.age
        return None


    def to_representation(self, instance):
        """Handle None or invalid decimal values gracefully."""
        representation = super().to_representation(instance)
        if representation.get('height') in [None, '', 'NaN']:
            representation['height'] = None  # Set to None or default value
        return representation
    
    def validate_skills(self, value):
        if not isinstance(value, list):
            raise serializers.ValidationError("Skills must be provided as a list.")
        return value
    
    def validate_url_link(self, value):
        # Custom validation for url_link
        if value and not value.startswith(("http://", "https://")):
            raise serializers.ValidationError("Invalid URL. Ensure it starts with 'http://' or 'https://'.")
        return value
    def validate_script_link(self, value):
        # Custom validation for script_link
        if value and not value.startswith(("http://", "https://")):
            raise serializers.ValidationError("Invalid URL. Ensure it starts with 'http://' or 'https://'.")
        return value 
    def get_profile_image(self, obj):
        profile = getattr(obj.user, 'profile', None)
        return profile.profile_image.url if profile and profile.profile_image else None

    def get_full_name(self, obj):
        return obj.user.full_name    

class CastingCallStatusSerializer(serializers.ModelSerializer):
    class Meta:
        model = CastingCall
        fields = ['status']

    def validate_status(self, value):
        if value not in dict(CastingCall.STATUS_CHOICES).keys():
            raise serializers.ValidationError("Invalid status.")
        return value
           
        
class FeedbackSerializer(serializers.ModelSerializer):
    class Meta:
        model = Feedback
        fields = ['id', 'project', 'user', 'feedback_text', 'rating', 'created_at']
        read_only_fields = ['id', 'created_at']

class HeadshotSerializer(serializers.ModelSerializer):
    class Meta:
        model = Headshot
        fields = ['headshot'] 


class ApprenticeshipSerializer(serializers.ModelSerializer):
    # skills_required = serializers.ListField(
    #     required=True
    # )
    skills_required = serializers.PrimaryKeyRelatedField(many=True, read_only=True)
    user_id = serializers.IntegerField(source='posted_by.id', read_only=True)
    profile_image = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()
    save_status = serializers.SerializerMethodField()
    applied_status = serializers.SerializerMethodField()
    age = serializers.SerializerMethodField()

    class Meta:
        model = Apprenticeship
        fields = ['id',
            'user_id','full_name','profile_image',
            'fullname',
            'organisation_or_individual',
            'profile_title',
            'email',
            'job_responsibilities',
            'location',
            'apprenticeship_type',
            'start_date',
            'end_date',
            'duration',
            'salary',
            'gender',
            'education_required',
            'skills_required',
            'skills_experience',
            'posted_by',
            'is_active',
            'created_at',
            'thumbnail',
            'logo',
            'save_status',  # Newly added
            'applied_status',  # Newly added
            'age',
        ]

    def get_age(self, obj):
        profile = getattr(obj.posted_by, 'profile', None)
        return profile.age if profile and profile.age else None

    def get_save_status(self, obj):
        user = self.context.get('request').user
        return SavedApprenticeship.objects.filter(apprenticeship_id=obj.id, user_id=user.id).exists()

    def get_applied_status(self, obj):
        user = self.context.get('request').user
        return ApprenticeApplication.objects.filter(apprenticeship=obj.id, user=user.id).exists()

    def get_profile_image(self, obj):
        profile = getattr(obj.posted_by, 'profile', None)
        return profile.profile_image.url if profile and profile.profile_image else None


    def get_full_name(self, obj):
        return obj.posted_by.full_name

    def validate_skills_required(self, value):
        if not value:
            raise serializers.ValidationError("Skills cannot be empty.")

        valid_skills = Skill.objects.filter(id__in=value[0])
        if len(valid_skills) != len(value[0]):
            raise serializers.ValidationError("Some of the skills are invalid.")

        return value

    def validate(self, data):
        start_date = data.get('start_date', getattr(self.instance, 'start_date', None))
        end_date = data.get('end_date', getattr(self.instance, 'end_date', None))
        skills_required = data.get('skills_required', [])
        skills_experience = data.get('skills_experience', getattr(self.instance, 'skills_experience', []))


        if start_date and end_date:
            if start_date >= end_date:
                raise serializers.ValidationError("End date must be after start date.")

            # expected_duration = (end_date.year - start_date.year) * 12 + (end_date.month - start_date.month)
            # duration = data.get('duration', getattr(self.instance, 'duration', None))
            # if duration and duration != expected_duration:
            #     raise serializers.ValidationError(
            #         f"Duration does not match the difference between start_date and end_date. Expected: {expected_duration} months."
            #     )

    
        if not skills_required:
            attribute = getattr(self.instance, 'skills_required', [])
            count = attribute.count()
            if count != len(skills_experience):
                raise serializers.ValidationError(
                    "The lengths of 'skills_required' and 'skills_experience' must match."
                )
            return data
        
        elif len(skills_required[0]) != len(skills_experience):
            raise serializers.ValidationError(
                "The lengths of 'skills_required' and 'skills_experience' must match."
            )

        return data


    def create(self, validated_data):
        skills = validated_data.pop('skills_required')
        apprenticeship = Apprenticeship.objects.create(**validated_data)
        apprenticeship.is_active = True
        apprenticeship.save()
        for skill in skills:
            apprenticeship.skills_required.set(skill)
        return apprenticeship

    def update(self, instance, validated_data):
        skills = validated_data.pop('skills_required', None)
        for attr, value in validated_data.items():
            setattr(instance, attr, value)
        instance.save()
        if skills is not None:
            for skill in skills:
                instance.skills_required.set(skill)
        return instance
    
class ApprenticeshipEditSerializer(serializers.ModelSerializer):
    skills_required = serializers.ListField(
        child=serializers.IntegerField(), required=False, allow_empty=True
    )
    skills_experience = serializers.ListField(
        child=serializers.CharField(), required=False, allow_empty=True
    )

    class Meta:
        model = Apprenticeship
        fields = [
            'fullname', 'organisation_or_individual', 'profile_title', 'email',
            'job_responsibilities', 'designation', 'location', 'apprenticeship_type',
            'start_date', 'end_date', 'duration', 'salary', 'gender',
            'education_required', 'age_required', 'skills_required', 'skills_experience',
            'is_active', 'status', 'thumbnail', 'logo', 'language', 'mobile_number'
        ]

    def validate(self, data):
        skills_required = data.get('skills_required', [])
        skills_experience = data.get('skills_experience', [])

        if len(skills_required) != len(skills_experience):
            raise serializers.ValidationError(
                "The length of `skills_required` and `skills_experience` must match."
            )
        return data

    def update(self, instance, validated_data):
        skills_required = validated_data.pop('skills_required', None)
        skills_experience = validated_data.pop('skills_experience', None)

        # Update basic fields
        for attr, value in validated_data.items():
            setattr(instance, attr, value)

        # Update skills
        if skills_required is not None:
            skill_objects = Skill.objects.filter(id__in=skills_required)
            instance.skills_required.set(skill_objects)

        if skills_experience is not None:
            instance.skills_experience = skills_experience

        instance.save()
        return instance

class ApprenticeshipViewSerializer(serializers.ModelSerializer):
    skills_required = serializers.SerializerMethodField()
    left_days = serializers.SerializerMethodField()
    applications_count = serializers.IntegerField(read_only=True)

    class Meta:
        model = Apprenticeship
        fields = [
            'id',
            'fullname',
            'organisation_or_individual',
            'profile_title',
            'email',
            'job_responsibilities',
            'location',
            'apprenticeship_type',
            'start_date',
            'end_date',
            'duration',
            'salary',
            'gender',
            'education_required',
            'skills_required',
            'skills_experience',
            'is_active',
            'created_at',
            'status',
            'applications_count',
            'left_days',
            'designation',
            'logo',
            'thumbnail',
            'age_required',
            'language',
            'mobile_number',
            
        ]

    def get_skills_required(self, obj):
        return [skill.name for skill in obj.skills_required.all()]
    
    def get_left_days(self, obj):
        if obj.end_date:
            days_left = (obj.end_date - date.today()).days
            return days_left if days_left >= 0 else 0
        return None


class AppliedApprenticeshipSerializer(serializers.ModelSerializer):
    # apprenticeship_details = serializers.SerializerMethodField()
    apprenticeship_details = ApprenticeshipSerializer(source='apprenticeship', read_only=True)

    class Meta:
        model = ApprenticeApplication
        fields = [
            'id', 'apprenticeship_details', 'status', 'applied_at','description'
        ]

    def get_apprenticeship_details(self, obj):
        return {
            "id": obj.apprenticeship.id,
            "profile_title": obj.apprenticeship.profile_title,
            "location": obj.apprenticeship.location,
            "apprenticeship_type": obj.apprenticeship.apprenticeship_type,
            "start_date": obj.apprenticeship.start_date,
            "end_date": obj.apprenticeship.end_date,
            "duration": obj.apprenticeship.duration,
            "salary": obj.apprenticeship.salary,
            "skills": [skill.name for skill in obj.apprenticeship.skills_required.all()],
        }


class ApprenticeshipApplicationStatusUpdateSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = ApprenticeApplication
        fields = ['status']

    def validate_status(self, value):
        valid_statuses = ['applied', 'selected', 'waiting_list', 'save_later', 'rejected']
        if value not in valid_statuses:
            raise serializers.ValidationError("Invalid status value.")
        
        instance = getattr(self, 'instance', None)
        if instance:
            if instance.status in ['selected', 'rejected']:
                raise serializers.ValidationError(f"Status '{instance.status}' cannot be changed once set.")
        return value
    

class ApprenticeshipApplicationSerializer(serializers.ModelSerializer):
    class Meta:
        model = ApprenticeApplication
        fields = ['apprenticeship', 'description']

    def validate_apprenticeship(self, value):
        # Check if the apprenticeship exists and is active
        if not value.is_active:
            raise serializers.ValidationError("This apprenticeship is no longer active.")
        return value

    def validate(self, data):
        user = self.context['request'].user
        apprenticeship = data['apprenticeship']

        # Ensure the user has not already applied for this apprenticeship
        if ApprenticeApplication.objects.filter(user=user, apprenticeship=apprenticeship).exists():
            raise serializers.ValidationError("You have already applied for this apprenticeship.")
        return data
    
class UserApplicationSerializer(serializers.ModelSerializer):
    user_id = serializers.SerializerMethodField()
    username = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()
    user_image = serializers.SerializerMethodField()
    location = serializers.SerializerMethodField()
    designation = serializers.SerializerMethodField()
    age_required = serializers.SerializerMethodField()
    thumbnail = serializers.SerializerMethodField()
    age = serializers.SerializerMethodField()

    class Meta:
        model = ApprenticeApplication
        fields = ['user_id', 'username', 'full_name','user_image','age', 'description', 'status', 'location', 'designation', 'age_required', 'thumbnail']

    def get_user_id(self, obj):
        return obj.user.id

    def get_username(self, obj):
        return obj.user.username
    
    def get_full_name(self, obj):
        return obj.user.full_name
    
    def get_age(self, obj):
        profile = Profile.objects.filter(user_id=obj.user).first()
        if profile:
            return profile.age if profile.age else None
        return None

    def get_user_image(self, obj):
        # Assuming the User model has a `profile_image` field
        creator_profile = get_object_or_404(Profile, user = obj.user)
        return creator_profile.profile_image.url if creator_profile.profile_image else None
    
    def get_location(self, obj):
        return obj.apprenticeship.location

    def get_designation(self, obj):
        return obj.apprenticeship.designation

    def get_age_required(self, obj):
        return obj.apprenticeship.age_required

    def get_thumbnail(self, obj):
        request = self.context.get('request')  # Fetch the request context for full URL
        if obj.apprenticeship.thumbnail:
            return request.build_absolute_uri(obj.apprenticeship.thumbnail.url) if request else obj.apprenticeship.thumbnail.url
        return None


class ApprenticeshipApplicationStatusUpdateSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = ApprenticeApplication
        fields = ['status']

    def validate_status(self, value):
        valid_statuses = ['selected', 'waiting', 'saved', 'rejected']
        if value not in valid_statuses:
            raise serializers.ValidationError("Invalid status value.")
        
        instance = getattr(self, 'instance', None)
        if instance:
            if instance.status in ['selected', 'rejected']:
                raise serializers.ValidationError(f"Status '{instance.status}' cannot be changed once set.")
        return value

class ApprenticeshipStatusSerializer(serializers.ModelSerializer):
    class Meta:
        model = Apprenticeship
        fields = ['status']

    def validate_status(self, value):
        if value not in dict(Apprenticeship.STATUS_CHOICES).keys():
            raise serializers.ValidationError("Invalid status.")
        return value



## internship serializers
    # skills_required = serializers.PrimaryKeyRelatedField(many=True, read_only=True)
class InternshipSerializer(serializers.ModelSerializer):
    # skills_required = serializers.ListField(
    #     required=True
    # )
    skills_required = serializers.PrimaryKeyRelatedField(many=True, read_only=True)
    user_id = serializers.IntegerField(source='posted_by.id', read_only=True)
    profile_image = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()
    applied_status = serializers.SerializerMethodField()
    save_status = serializers.SerializerMethodField()
    age = serializers.SerializerMethodField()

    # skills_required = serializers.ListField(
    #     child=serializers.IntegerField(), required=True
    # )
    # skills_experience = serializers.ListField(
    #     child=serializers.CharField(), required=True
    # )

    class Meta:
        model = Internship
        fields = [
            'id',
            'user_id','full_name','profile_image',
            'fullname',
            'organisation_or_individual',
            'profile_title',
            'email',
            'job_responsibilities',
            'location',
            'internship_type',
            'start_date',
            'end_date',
            'duration',
            'salary',
            'gender',
            'education_required',
            'skills_required',
            'skills_experience',
            'posted_by',
            'is_active',
            'created_at',
            'thumbnail',
            'logo',
            'applied_status',
            'save_status',
            'age'
        ]

    def get_age(self, obj):
        profile = getattr(obj.posted_by, 'profile', None)
        return profile.age if profile and profile.age else None

    def get_applied_status(self, obj):
        # Check if the internship is applied by the current user
        user = self.context.get('request').user
        if user.is_authenticated:
            return InternshipApplication.objects.filter(internship=obj, user=user).exists()
        return False

    def get_save_status(self, obj):
        # Check if the internship is saved by the current user
        user = self.context.get('request').user
        if user.is_authenticated:
            return SavedInternship.objects.filter(internship=obj, user=user).exists()
        return False

    def get_profile_image(self, obj):
        profile = getattr(obj.posted_by, 'profile', None)
        return profile.profile_image.url if profile and profile.profile_image else None


    def get_full_name(self, obj):
        return obj.posted_by.full_name

    def validate_skills_required(self, value):
        if not value:
            raise serializers.ValidationError("Skills cannot be empty.")

        valid_skills = Skill.objects.filter(id__in=value[0])
        if len(valid_skills) != len(value[0]):
            raise serializers.ValidationError("Some of the skills are invalid.")

        return value

    def validate(self, data):
        start_date = data.get('start_date', getattr(self.instance, 'start_date', None))
        end_date = data.get('end_date', getattr(self.instance, 'end_date', None))
        skills_required = data.get('skills_required', [])
        skills_experience = data.get('skills_experience', getattr(self.instance, 'skills_experience', []))


        if start_date and end_date:
            if start_date >= end_date:
                raise serializers.ValidationError("End date must be after start date.")

            # expected_duration = (end_date.year - start_date.year) * 12 + (end_date.month - start_date.month)
            # duration = data.get('duration', getattr(self.instance, 'duration', None))
            # if duration and duration != expected_duration:
            #     raise serializers.ValidationError(
            #         f"Duration does not match the difference between start_date and end_date. Expected: {expected_duration} months."
            #     )

    
        
        if not skills_required:
            attribute = getattr(self.instance, 'skills_required', [])
            count = attribute.count()
            if count != len(skills_experience):
                raise serializers.ValidationError(
                    "The lengths of 'skills_required' and 'skills_experience' must match."
                )
            return data
        
        elif len(skills_required[0]) != len(skills_experience):
            raise serializers.ValidationError(
                "The lengths of 'skills_required' and 'skills_experience' must match."
            )

        return data


    def create(self, validated_data):
        skills = validated_data.pop('skills_required')
        internship = Internship.objects.create(**validated_data)
        internship.is_active = True
        internship.save()
        for skill in skills:
            internship.skills_required.set(skill)
        return internship

    def update(self, instance, validated_data):
        skills = validated_data.pop('skills_required', None)
        for attr, value in validated_data.items():
            setattr(instance, attr, value)
        instance.save()
        if skills is not None:
            for skill in skills:
                instance.skills_required.set(skill)
        return instance

# from rest_framework import serializers

class InternshipEditSerializer(serializers.ModelSerializer):
    skills_required = serializers.ListField(
        child=serializers.IntegerField(), required=False, allow_empty=True
    )
    skills_experience = serializers.ListField(
        child=serializers.CharField(), required=False, allow_empty=True
    )

    class Meta:
        model = Internship
        fields = [
            'fullname', 'organisation_or_individual', 'profile_title', 'email',
            'job_responsibilities', 'designation', 'location', 'internship_type',
            'start_date', 'end_date', 'duration', 'salary', 'gender',
            'education_required', 'age_required', 'skills_required', 'skills_experience',
            'is_active', 'status', 'thumbnail', 'logo', 'language', 'mobile_number'
        ]

    def validate(self, data):
        skills_required = data.get('skills_required', [])
        skills_experience = data.get('skills_experience', [])

        if len(skills_required) != len(skills_experience):
            raise serializers.ValidationError(
                "The length of `skills_required` and `skills_experience` must match."
            )
        return data

    def update(self, instance, validated_data):
        # Handle related fields like skills_required separately
        skills_required = validated_data.pop('skills_required', None)
        skills_experience = validated_data.pop('skills_experience', None)

        # Update basic fields
        for attr, value in validated_data.items():
            setattr(instance, attr, value)

        # Update skills
        if skills_required is not None:
            skill_objects = Skill.objects.filter(id__in=skills_required)
            instance.skills_required.set(skill_objects)

        if skills_experience is not None:
            instance.skills_experience = skills_experience

        instance.save()
        return instance


class InternshipViewSerializer(serializers.ModelSerializer):
    skills_required = serializers.SerializerMethodField()
    left_days = serializers.SerializerMethodField()
    applications_count = serializers.IntegerField(read_only=True)

    class Meta:
        model = Internship
        fields = [
            'id',
            'fullname',
            'organisation_or_individual',
            'profile_title',
            'email',
            'job_responsibilities',
            'location',
            'internship_type',
            'start_date',
            'end_date',
            'duration',
            'salary',
            'gender',
            'education_required',
            'skills_required',
            'skills_experience',
            'is_active',
            'created_at',
            'status',
            'left_days',
            'applications_count',
            'designation',
            'logo',
            'thumbnail',
            'age_required',
            'language',
            'mobile_number',
        ]

    def get_skills_required(self, obj):
        return [skill.name for skill in obj.skills_required.all()]
    
    def get_left_days(self, obj):
        if obj.end_date:
            return (obj.end_date - date.today()).days
        return None


class AppliedInternshipSerializer(serializers.ModelSerializer):
    # internship_details = serializers.SerializerMethodField()
    internship_details = InternshipSerializer(source='internship', read_only=True)

    class Meta:
        model = InternshipApplication
        fields = [
            'id', 'internship_details', 'status', 'applied_at'
        ]

    def get_internship_details(self, obj):
        return {
            "id": obj.internship.id,
            "profile_title": obj.internship.profile_title,
            "location": obj.internship.location,
            "internship_type": obj.internship.internship_type,
            "start_date": obj.internship.start_date,
            "end_date": obj.internship.end_date,
            "duration": obj.internship.duration,
            "salary": obj.internship.salary,
            "skills": [skill.name for skill in obj.internship.skills_required.all()],
        }


class InternshipApplicationStatusUpdateSerializer(serializers.ModelSerializer):
    
    class Meta:
        model = InternshipApplication
        fields = ['status']

    def validate_status(self, value):
        valid_statuses = ['applied', 'selected', 'waiting_list', 'save_later', 'rejected']
        if value not in valid_statuses:
            raise serializers.ValidationError("Invalid status value.")
        
        instance = getattr(self, 'instance', None)
        if instance:
            if instance.status in ['selected', 'rejected']:
                raise serializers.ValidationError(f"Status '{instance.status}' cannot be changed once set.")
        return value
    

class InternshipApplicationSerializer(serializers.ModelSerializer):
    class Meta:
        model = InternshipApplication
        fields = ['internship', 'description']

    def validate_internship(self, value):
        # Check if the internship exists and is active
        if not value.is_active:
            raise serializers.ValidationError("This internship is no longer active.")
        return value

    def validate(self, data):
        user = self.context['request'].user
        internship = data['internship']

        # Ensure the user has not already applied for this internship
        if InternshipApplication.objects.filter(user=user, internship=internship).exists():
            raise serializers.ValidationError("You have already applied for this internship.")
        return data
    
class UserInternshipApplicationSerializer(serializers.ModelSerializer):
    user_id = serializers.SerializerMethodField()
    username = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()
    user_image = serializers.SerializerMethodField()
    location = serializers.SerializerMethodField()
    designation = serializers.SerializerMethodField()
    age_required = serializers.SerializerMethodField()
    thumbnail = serializers.SerializerMethodField()

    class Meta:
        model = InternshipApplication
        fields = ['user_id', 'username', 'full_name','user_image', 'description', 'description', 'status', 'location', 'designation', 'age_required','thumbnail']

    def get_user_id(self, obj):
        return obj.user.id

    def get_username(self, obj):
        return obj.user.username
    
    def get_full_name(self, obj):
        return obj.user.full_name

    def get_user_image(self, obj):
        # Assuming the User model has a `profile_image` field
        creator_profile = get_object_or_404(Profile, user = obj.user)
        return creator_profile.profile_image.url if creator_profile.profile_image else None
    def get_location(self, obj):
        return obj.internship.location

    def get_designation(self, obj):
        return obj.internship.designation

    def get_age_required(self, obj):
        return obj.internship.age_required
    
    def get_thumbnail(self, obj):
        request = self.context.get('request')  # Fetch the request context for full URL
        if obj.internship.thumbnail:
            return request.build_absolute_uri(obj.internship.thumbnail.url) if request else obj.internship.thumbnail.url
        return None

class InternshipStatusSerializer(serializers.ModelSerializer):
    class Meta:
        model = Internship
        fields = ['status']

    def validate_status(self, value):
        if value not in dict(Internship.STATUS_CHOICES).keys():
            raise serializers.ValidationError("Invalid status.")
        return value
